@@ -31,44 +31,44 @@ extern void c_quadform_vMu(gsl_vector *v,
 extern double c_det(gsl_matrix *A);
 
 extern void updateRPus(int *q_adj,
-                     gsl_matrix *Y,
-                     gsl_matrix *X,
-                     gsl_matrix *B,
-                     gsl_matrix *gamma,
-                     gsl_matrix *Sigma,
-                     gsl_matrix *invSigma,
-                     gsl_matrix *updateNonzB,
-                     gsl_vector *beta0,
-                     gsl_vector *v,
-                     gsl_vector *omega,
-                     double eta,
-                     gsl_matrix *rwbetaVar,
-                     gsl_matrix *accept_B);
+                       gsl_matrix *Y,
+                       gsl_matrix *X,
+                       gsl_matrix *B,
+                       gsl_matrix *gamma,
+                       gsl_matrix *Sigma,
+                       gsl_matrix *invSigma,
+                       gsl_matrix *updateNonzB,
+                       gsl_vector *beta0,
+                       gsl_vector *v,
+                       gsl_vector *omega,
+                       double eta,
+                       gsl_matrix *rwbetaVar,
+                       gsl_matrix *accept_B);
 
 extern void updateIPus(gsl_matrix *Y,
-                     gsl_matrix *X,
-                     gsl_matrix *B,
-                     gsl_matrix *Sigma,
-                     gsl_matrix *invSigma,
-                     gsl_vector *beta0,
-                     gsl_vector *mu0,
-                     double h0);
+                       gsl_matrix *X,
+                       gsl_matrix *B,
+                       gsl_matrix *Sigma,
+                       gsl_matrix *invSigma,
+                       gsl_vector *beta0,
+                       gsl_vector *mu0,
+                       double h0);
 
 extern void updateCPus(int *q_adj,
-                     gsl_matrix *Y,
-                     gsl_matrix *X,
-                     gsl_matrix *B,
-                     gsl_matrix *gamma,
-                     gsl_matrix *Sigma,
-                     gsl_matrix *invSigma,
-                     gsl_vector *beta0,
-                     gsl_vector *omega,
-                     double eta,
-                     gsl_matrix *Psi0,
-                     double rho0,
-                     gsl_matrix *Psi_prop,
-                     double rho_prop,
-                     int *accept_Sigma);
+                       gsl_matrix *Y,
+                       gsl_matrix *X,
+                       gsl_matrix *B,
+                       gsl_matrix *gamma,
+                       gsl_matrix *Sigma,
+                       gsl_matrix *invSigma,
+                       gsl_vector *beta0,
+                       gsl_vector *omega,
+                       double eta,
+                       gsl_matrix *Psi0,
+                       double rho0,
+                       gsl_matrix *Psi_prop,
+                       double rho_prop,
+                       int *accept_Sigma);
 
 extern void c_rigamma(double *sample,
                       double alpha,
@@ -85,9 +85,9 @@ extern void c_dmvnorm(gsl_vector *x,
                       double     *value);
 
 extern double sumCorus_j(gsl_matrix *Sigma,
-                       gsl_matrix *gamma,
-                       int j,
-                       int k);
+                         gsl_matrix *gamma,
+                         int j,
+                         int k);
 
 extern double logit(double val);
 
@@ -97,5 +97,6 @@ extern double one_invLogit(double val);
 
 extern void c_riwishart(double v,
                         gsl_matrix *X_ori,
-                        gsl_matrix *sample);
+                        gsl_matrix *sample,
+                        gsl_matrix *sampleInv);
 
