@@ -938,7 +938,7 @@ void mzipBVS_general_update_SigmaV(gsl_matrix *V,
     
     df_ = rho0 + (double) n;
     
-    c_riwishart(df_, Sum, SigmaV);
+    c_riwishart(df_, Sum, SigmaV, invSigmaV);
     
     gsl_matrix_free(VV);
     gsl_matrix_free(Sum);
@@ -978,6 +978,7 @@ void mzipBVS_general_update_R_Gibbs(gsl_matrix *X0,
     gsl_matrix *ee = gsl_matrix_calloc(q, q);
     gsl_matrix *Sum = gsl_matrix_calloc(q, q);
     gsl_matrix *Sigma = gsl_matrix_calloc(q, q);
+    gsl_matrix *invSigma = gsl_matrix_calloc(q, q);
     gsl_matrix *scaled_k0k0T = gsl_matrix_calloc(q, q);
 
     
@@ -1025,7 +1026,7 @@ void mzipBVS_general_update_R_Gibbs(gsl_matrix *X0,
     gsl_matrix_add(Sum, scaled_k0k0T);
     
     df_ = (double) n + 1;
-    c_riwishart2(df_, Sum, Sigma);
+    c_riwishart2(df_, Sum, Sigma, invSigma);
     
     for(i = 0; i < q; i++)
     {
@@ -1047,6 +1048,7 @@ void mzipBVS_general_update_R_Gibbs(gsl_matrix *X0,
     gsl_matrix_free(ee);
     gsl_matrix_free(Sum);
     gsl_matrix_free(Sigma);
+    gsl_matrix_free(invSigma);
     gsl_matrix_free(scaled_k0k0T);
     
     gsl_vector_free(kappa);
@@ -1085,118 +1087,119 @@ void mzipBVS_general_update_V(gsl_matrix *Y,
     int q = Y -> size2;
     int p1 = X1 -> size2;
     
-    jj = (int) runif(0, q);
-    
     gsl_vector *beta = gsl_vector_calloc(p1);
     gsl_vector *Xbeta = gsl_vector_calloc(n);
     
-    for(k = 0; k < p1; k++)
+    for(jj=0; jj<q; jj++)
     {
-        gsl_vector_set(beta, k, gsl_matrix_get(B, k, jj));
-    }
-    gsl_blas_dgemv(CblasNoTrans, 1, X1, beta, 0, Xbeta);
-    
-    
-    for(i=0; i<n; i++)
-    {
-        loglh = 0;
-        loglh_prop = 0;
-        D1 = 0; D2 = 0;
-        D1_prop = 0; D2_prop = 0;
-        
-        tempB = 0;
-        if(gsl_matrix_get(W, i, jj) >= 0)
+        for(k = 0; k < p1; k++)
         {
-            tempB = gsl_vector_get(beta0, jj)+gsl_vector_get(Xbeta, i)+gsl_matrix_get(V, i, jj)+log(gsl_vector_get(xi, i));
-            
-            loglh += gsl_matrix_get(V, i, jj)*gsl_matrix_get(Y, i, jj) - exp(tempB);
-            D1 += gsl_matrix_get(Y, i, jj) - exp(tempB);
-            D2 -= exp(tempB);
+            gsl_vector_set(beta, k, gsl_matrix_get(B, k, jj));
         }
+        gsl_blas_dgemv(CblasNoTrans, 1, X1, beta, 0, Xbeta);
         
-        loglh -= 0.5 * pow(gsl_matrix_get(V, i, jj), 2) * gsl_matrix_get(invSigmaV, jj, jj);
-        D1 -= gsl_matrix_get(V, i, jj)* gsl_matrix_get(invSigmaV, jj, jj);
-        D2 -= gsl_matrix_get(invSigmaV, jj, jj);
         
-        for(k = 0; k < q; k++)
+        for(i=0; i<n; i++)
         {
-            if(k != jj)
-            {
-                loglh -= gsl_matrix_get(V, i, jj) * gsl_matrix_get(V, i, k) * gsl_matrix_get(invSigmaV, jj, k);
-                D1 -= gsl_matrix_get(V, i, k) * gsl_matrix_get(invSigmaV, jj, k);
-            }
-        }
-        
-        if(D1/D2 > 1 || D1/D2 < -1 )
-        {
-            temp_prop =  rnorm(gsl_matrix_get(V, i, jj), sqrt(V_prop_var));
+            loglh = 0;
+            loglh_prop = 0;
+            D1 = 0; D2 = 0;
+            D1_prop = 0; D2_prop = 0;
             
-            tempB_prop = 0;
+            tempB = 0;
             if(gsl_matrix_get(W, i, jj) >= 0)
             {
-                tempB_prop = gsl_vector_get(beta0, jj)+gsl_vector_get(Xbeta, i)+temp_prop+log(gsl_vector_get(xi, i));
+                tempB = gsl_vector_get(beta0, jj)+gsl_vector_get(Xbeta, i)+gsl_matrix_get(V, i, jj)+log(gsl_vector_get(xi, i));
                 
-                loglh_prop += temp_prop*gsl_matrix_get(Y, i, jj) - exp(tempB_prop);
+                loglh += gsl_matrix_get(V, i, jj)*gsl_matrix_get(Y, i, jj) - exp(tempB);
+                D1 += gsl_matrix_get(Y, i, jj) - exp(tempB);
+                D2 -= exp(tempB);
             }
-            loglh_prop -= 0.5 * pow(temp_prop, 2) * gsl_matrix_get(invSigmaV, jj, jj);
+            
+            loglh -= 0.5 * pow(gsl_matrix_get(V, i, jj), 2) * gsl_matrix_get(invSigmaV, jj, jj);
+            D1 -= gsl_matrix_get(V, i, jj)* gsl_matrix_get(invSigmaV, jj, jj);
+            D2 -= gsl_matrix_get(invSigmaV, jj, jj);
             
             for(k = 0; k < q; k++)
             {
                 if(k != jj)
                 {
-                    loglh_prop -= temp_prop * gsl_matrix_get(V, i, k) * gsl_matrix_get(invSigmaV, jj, k);
+                    loglh -= gsl_matrix_get(V, i, jj) * gsl_matrix_get(V, i, k) * gsl_matrix_get(invSigmaV, jj, k);
+                    D1 -= gsl_matrix_get(V, i, k) * gsl_matrix_get(invSigmaV, jj, k);
                 }
             }
             
-            logR = loglh_prop - loglh;
-        }else
-        {
-            temp_prop_me    = gsl_matrix_get(V, i, jj) - D1/D2;
-            temp_prop_var   = -pow(2.4, 2)/D2;
-            
-            temp_prop =  rnorm(temp_prop_me, sqrt(temp_prop_var));
-            
-            tempB_prop = 0;
-            if(gsl_matrix_get(W, i, jj) >= 0)
+            if(D1/D2 > 1 || D1/D2 < -1 )
             {
-                tempB_prop = gsl_vector_get(beta0, jj)+gsl_vector_get(Xbeta, i)+temp_prop+log(gsl_vector_get(xi, i));
+                temp_prop =  rnorm(gsl_matrix_get(V, i, jj), sqrt(V_prop_var));
                 
-                loglh_prop += temp_prop*gsl_matrix_get(Y, i, jj) - exp(tempB_prop);
-                D1_prop += gsl_matrix_get(Y, i, jj) - exp(tempB_prop);
-                D2_prop -= exp(tempB_prop);
-            }
-            loglh_prop -= 0.5 * pow(temp_prop, 2) * gsl_matrix_get(invSigmaV, jj, jj);
-            D1_prop -= temp_prop * gsl_matrix_get(invSigmaV, jj, jj);
-            D2_prop -= gsl_matrix_get(invSigmaV, jj, jj);
-            
-            for(k = 0; k < q; k++)
-            {
-                if(k != jj)
+                tempB_prop = 0;
+                if(gsl_matrix_get(W, i, jj) >= 0)
                 {
-                    loglh_prop -= temp_prop * gsl_matrix_get(V, i, k) * gsl_matrix_get(invSigmaV, jj, k);
-                    D1_prop -= gsl_matrix_get(V, i, k) * gsl_matrix_get(invSigmaV, jj, k);
+                    tempB_prop = gsl_vector_get(beta0, jj)+gsl_vector_get(Xbeta, i)+temp_prop+log(gsl_vector_get(xi, i));
+                    
+                    loglh_prop += temp_prop*gsl_matrix_get(Y, i, jj) - exp(tempB_prop);
                 }
+                loglh_prop -= 0.5 * pow(temp_prop, 2) * gsl_matrix_get(invSigmaV, jj, jj);
+                
+                for(k = 0; k < q; k++)
+                {
+                    if(k != jj)
+                    {
+                        loglh_prop -= temp_prop * gsl_matrix_get(V, i, k) * gsl_matrix_get(invSigmaV, jj, k);
+                    }
+                }
+                
+                logR = loglh_prop - loglh;
+            }else
+            {
+                temp_prop_me    = gsl_matrix_get(V, i, jj) - D1/D2;
+                temp_prop_var   = -pow(2.4, 2)/D2;
+                
+                temp_prop =  rnorm(temp_prop_me, sqrt(temp_prop_var));
+                
+                tempB_prop = 0;
+                if(gsl_matrix_get(W, i, jj) >= 0)
+                {
+                    tempB_prop = gsl_vector_get(beta0, jj)+gsl_vector_get(Xbeta, i)+temp_prop+log(gsl_vector_get(xi, i));
+                    
+                    loglh_prop += temp_prop*gsl_matrix_get(Y, i, jj) - exp(tempB_prop);
+                    D1_prop += gsl_matrix_get(Y, i, jj) - exp(tempB_prop);
+                    D2_prop -= exp(tempB_prop);
+                }
+                loglh_prop -= 0.5 * pow(temp_prop, 2) * gsl_matrix_get(invSigmaV, jj, jj);
+                D1_prop -= temp_prop * gsl_matrix_get(invSigmaV, jj, jj);
+                D2_prop -= gsl_matrix_get(invSigmaV, jj, jj);
+                
+                for(k = 0; k < q; k++)
+                {
+                    if(k != jj)
+                    {
+                        loglh_prop -= temp_prop * gsl_matrix_get(V, i, k) * gsl_matrix_get(invSigmaV, jj, k);
+                        D1_prop -= gsl_matrix_get(V, i, k) * gsl_matrix_get(invSigmaV, jj, k);
+                    }
+                }
+                
+                temp_prop_me_prop   = temp_prop - D1_prop/D2_prop;
+                temp_prop_var_prop   = -pow(2.4, 2)/D2_prop;
+                
+                logProp_IniToProp = dnorm(temp_prop, temp_prop_me, sqrt(temp_prop_var), 1);
+                logProp_PropToIni = dnorm(gsl_matrix_get(V, i, jj), temp_prop_me_prop, sqrt(temp_prop_var_prop), 1);
+                
+                logR = loglh_prop - loglh + logProp_PropToIni - logProp_IniToProp;
             }
+                        
+            u = log(runif(0, 1)) < logR;
             
-            temp_prop_me_prop   = temp_prop - D1_prop/D2_prop;
-            temp_prop_var_prop   = -pow(2.4, 2)/D2_prop;
-            
-            logProp_IniToProp = dnorm(temp_prop, temp_prop_me, sqrt(temp_prop_var), 1);
-            logProp_PropToIni = dnorm(gsl_matrix_get(V, i, jj), temp_prop_me_prop, sqrt(temp_prop_var_prop), 1);
-            
-            logR = loglh_prop - loglh + logProp_PropToIni - logProp_IniToProp;
-        }
-        
-        
-        u = log(runif(0, 1)) < logR;
-        
-        if(u == 1)
-        {
-            gsl_matrix_set(V, i, jj, temp_prop);
-            gsl_matrix_set(accept_V, i, jj, gsl_matrix_get(accept_V, i, jj)+1);
+            if(u == 1)
+            {
+                gsl_matrix_set(V, i, jj, temp_prop);
+                gsl_matrix_set(accept_V, i, jj, gsl_matrix_get(accept_V, i, jj)+1);
+            }
         }
     }
     
+    
     gsl_vector_free(beta);
     gsl_vector_free(Xbeta);
     
@@ -1903,7 +1906,7 @@ void mzipBVS_general_update_sigSq_alpha(gsl_matrix *A,
     
     for(j = 0; j < q; j++)
     {
-        zeta_rate += pow(gsl_vector_get(alpha, j), 2)/gsl_vector_get(v_alpha, j);
+        zeta_rate += pow(gsl_vector_get(alpha, j), 2)/pow(gsl_vector_get(v_alpha, j), 2);
     }
     zeta_rate /= 2;
     zeta_rate += gsl_vector_get(b_alpha, kk);
@@ -1957,7 +1960,7 @@ void mzipBVS_general_update_sigSq_beta(gsl_matrix *B,
     {
         if(gsl_matrix_get(gamma_beta, kk, j) == 1)
         {
-            zeta_rate += pow(gsl_vector_get(beta, j), 2)/gsl_vector_get(v_beta, j);
+            zeta_rate += pow(gsl_vector_get(beta, j), 2)/pow(gsl_vector_get(v_beta, j),2);
         }
     }
     zeta_rate /= 2;
